/***************************************************************************
**                                                                        **
**  The Mathematical & Engineering Components (ME.com) library is         **
**  a original C++ framework for mathematics & statistical data analysis  **
**  Copyright (C) 2015 Igor Gaidyshev                                     **
**                                                                        **
**  This program is free software: you can redistribute it and/or modify  **
**  it under the terms of the GNU General Public License as published by  **
**  the Free Software Foundation, either version 3 of the License, or     **
**  (at your option) any later version.                                   **
**                                                                        **
**  This program is distributed in the hope that it will be useful,       **
**  but WITHOUT ANY WARRANTY; without even the implied warranty of        **
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         **
**  GNU General Public License for more details.                          **
**                                                                        **
**  You should have received a copy of the GNU General Public License     **
**  along with this program. If not, see http://www.gnu.org/licenses      **
**                                                                        **
****************************************************************************
**           Author: Igor Gaidyshev                                       **
**          Contact: igor.gaidyshev@gmail.com                             **
***************************************************************************/

#include <math.h>

///////////////////
// Custom constants
#define LOG_2 0.693147180559945
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/////////////////////
// Custom definitions
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define ABS(a) ((a) >= 0 ? (a) : (-a))
#define SIGN(a) ((a) > 0 ? 1 : -1)

typedef double (*userFunc) (double);
typedef double (*userFuncRight) (double*,long,double*,long);
typedef void (*userFuncGrad) (double*,long,double*,double*,long,double,userFuncRight);

//////////////////////////////////////////////////
// To compile DLL for EXCEL uncomment string below
#define EXCEL

#ifndef EXCEL
  #define MODIFICATOR
#else
  #define MODIFICATOR __stdcall
  extern "C"
   {
#endif

/////////////////////////
// Prototype of functions

////////////////////////////////
// The function for outside call
double MODIFICATOR A1 (double);
double MODIFICATOR Abbe (double*,long);
double MODIFICATOR AbbeDistribution(double,long);
double MODIFICATOR Aberration (double*,long);
double MODIFICATOR AndersonDarlingTest (double*,long,double*);
double MODIFICATOR VectorSum (double*,long);
double MODIFICATOR ANOVA (double*,long*,long);
double MODIFICATOR AnsariBradleyTest (double*,long,double*,long,double*,long);
double MODIFICATOR Arsh (double);
double MODIFICATOR Asinh (double*,long);
double MODIFICATOR AsinhML (double*,long,double);
double MODIFICATOR AsinhTransform (double,double);
double MODIFICATOR Atanh(double);
double MODIFICATOR BarnardProbability (long,long,long,long,double,double*);
double MODIFICATOR BarnardTest (double,double,double,double,double*,double*);
double MODIFICATOR BarnardTestProb (long,long,double,double,double*);
double MODIFICATOR Bartlett (double*,long*,long);
double MODIFICATOR BDistribution (double,double,double);
double MODIFICATOR Bernoulli (long,double);
double MODIFICATOR Bessel (double,double);
double MODIFICATOR Beta (double,double);
double MODIFICATOR Bhapkar (double*,long*,long);
double MODIFICATOR BhapkarDeshpande (double*,long*,long);
double MODIFICATOR BhapkarTest (double*,long);
double MODIFICATOR BinomialDensity(long,long,double);
double MODIFICATOR BinomialDistribution(long,long,double);
double MODIFICATOR BowkerTest (double*,long);
double MODIFICATOR BoxCox (double*,long);
double MODIFICATOR BoxCoxML (double*,long,double);
double MODIFICATOR BoxCoxTransform (double,double);
double MODIFICATOR Tetra (double*,double*,long);
double MODIFICATOR TetraVariance (double*,double*,long);
double MODIFICATOR Brown (double*,long*,long);
double MODIFICATOR ChebyshevHermite (long,double);
double MODIFICATOR ChenTest (double*,long,double*,double*);
double MODIFICATOR ChiSquare(double*,long,long);
double MODIFICATOR ChiSquareDistribution (double,double);
double MODIFICATOR ChiSquareDistributionNC (double,double,double);
double MODIFICATOR ChiSquareFisherTest (double*,double*,double*,long,double*);
double MODIFICATOR CityBlock (double*,double*,long);
double MODIFICATOR CochranCoxTest (double*,long,double*,long,double*);
double MODIFICATOR CochraneG (double*,long,long);
double MODIFICATOR CochraneQ (double*,long,long);
double MODIFICATOR CochranGDistribution (double,double,double,long);
double MODIFICATOR Concord (double*,long,long);
double MODIFICATOR ConoverTest (double*,long,double*,long,double*);
double MODIFICATOR Cov (double*,double*,long);
double MODIFICATOR CoxTest (double*,double*,long,double*,double*,long,double*);
double MODIFICATOR Cramer(double*,long,long,double*);
double MODIFICATOR CramerTest (double*,long,double*,long,double*);
double MODIFICATOR CramerVonMisesTest (double*,long,double*);
double MODIFICATOR CressieRead(double*,long,long,double);
double MODIFICATOR DAgostinoTest (double*,long,double*);
double MODIFICATOR KendallCI(double,double,double,double);
double MODIFICATOR Deshpande (double*,long*,long);
double MODIFICATOR DispersionEntropy (double*,long,long,double);
double MODIFICATOR DunnDistribution (double,long);
double MODIFICATOR Entropy (double*,long,double);
double MODIFICATOR EppsPulleyTest (double*,long,double*);
double MODIFICATOR Euclid (double*,double*,long);
double MODIFICATOR EuclidNorma (double*,long);
double MODIFICATOR EuclidNormaVector (double*,long);
double MODIFICATOR EuclidNormaWithoutDiag (double*,long);
double MODIFICATOR EuklidNorma0 (double*,long);
double MODIFICATOR FarrarGlauber1 (double*,long,long,double*,double*);
double MODIFICATOR FDistribution (double,double,double);
double MODIFICATOR Fechner (double*,double*,long);
double MODIFICATOR FisherProbability (double,long,long,long,long,double*);
double MODIFICATOR FisherTest (double,double,double,double,double*,double*);
double MODIFICATOR FreemanHaltonMC (double*,long,long,double);
double MODIFICATOR Friedman (double*,long,long,long);
double MODIFICATOR FTest (double*,long,double*,long,double*);
double MODIFICATOR GailMantel (double*,long,long);
double MODIFICATOR GammaAdd (double*,double*,long,double,double,double);
double MODIFICATOR GammaDistribution (double,double,double,double);
double MODIFICATOR GammaIncomplete(double,double);
double MODIFICATOR GammaInteger (double);
double MODIFICATOR GearyTest (double*,long,double*);
double MODIFICATOR GehanTest (double*,double*,long,double*,double*,long,double*);
double MODIFICATOR Gini (double*,long);
double MODIFICATOR GompertzAdd (double*,long,double,double,double);
double MODIFICATOR Haberman(double*,long,long);
double MODIFICATOR Hellinger(double*,long,long,double);
double MODIFICATOR HenzeZirklerTest (double*,long,long,double*);
double MODIFICATOR Hermite (long,double);
double MODIFICATOR Hotelling (double*,double*,long,long,long);
double MODIFICATOR InterRange (double*,long);
double MODIFICATOR InverseChiSquareDistribution (double,double);
double MODIFICATOR InverseChiSquareDistributionNC (double,double,double);
double MODIFICATOR InverseFDistribution (double,double,double);
double MODIFICATOR InverseNormalDistribution (double);
double MODIFICATOR InverseSerialDistribution (long,long,long);
double MODIFICATOR InverseSMMDistribution(double,double,double);
double MODIFICATOR InverseTDistribution (double,double);
double MODIFICATOR Jaccard (double*,double*,long);
double MODIFICATOR JamesSue (double*,double*,long,long,long);
double MODIFICATOR JarqueBeraTest (double*,long,double*);
double MODIFICATOR JohnDraper (double*,long);
double MODIFICATOR JohnDraperML (double*,long,double);
double MODIFICATOR JohnDraperTransform (double,double);
double MODIFICATOR JonckheereTerpstra (double*,long*,long);
double MODIFICATOR Kappa (double,double,double,double,double*,double*,double);
double MODIFICATOR KlotzTest (double*,long,double*,long,double*);
double MODIFICATOR Kolmogorov (double*,long,double*);
double MODIFICATOR KolmogorovSimpleTest (double*,double,double,long,double*);
double MODIFICATOR KolmogorovTest (double*,long,double*);
double MODIFICATOR Krauth (double*,long,double);
double MODIFICATOR KruskalWallis (double*,long*,long,long);
double MODIFICATOR KuiperDistribution (double);
double MODIFICATOR KuiperExactTest (double*,long,double*,long,double*);
double MODIFICATOR KuiperExactTestAdd (double*,long,double*,long);
double MODIFICATOR Kullback (double*,double*,long,long,long);
double MODIFICATOR Kurtosis (double*,long);
double MODIFICATOR KurtosisTest (double*,long,double*);
double MODIFICATOR Laguerre (long,long,double);
double MODIFICATOR LambdaDistribution (double);
double MODIFICATOR Legendre (long,double);
double MODIFICATOR Levene (double*,long*,long);
double MODIFICATOR LikelihoodRatio(double*,long,long);
double MODIFICATOR LogEuler(double);
double MODIFICATOR LogFactorial (long);
double MODIFICATOR LogFactorialD (double);
double MODIFICATOR LogGamma (double);
double MODIFICATOR Logit (double);
double MODIFICATOR LogNormalDistribution (double,double,double);
double MODIFICATOR Mahalanobis (double*,double*,double*,long);
double MODIFICATOR Manly (double*,long);
double MODIFICATOR ManlyML (double*,long,double);
double MODIFICATOR ManlyTransform (double,double);
double MODIFICATOR MardiaKurtosisTest (double*,long,long,double*);
double MODIFICATOR MardiaSkewnessTest (double*,long,long,double*);
double MODIFICATOR MaxEntropy(long,double);
double MODIFICATOR Maximum(double*,long);
double MODIFICATOR McNemarDistribution (double,double);
double MODIFICATOR McNemarExact (double,double,double*);
double MODIFICATOR McNemarTest (double,double,long,double*);
double MODIFICATOR Mean (double*,long);
double MODIFICATOR Rate (double*,long);
double MODIFICATOR MedianTest (double*,long,double*,long,double*);
double MODIFICATOR Minimum(double*,long);
double MODIFICATOR Moment (double*,long,long);
double MODIFICATOR MoodBrownTest (double*,long,double*,long,double*);
double MODIFICATOR NormalDistribution (double);
double MODIFICATOR NormalIntegral (double);
double MODIFICATOR NPV (double,double,double,double,double*,double*,double,double);
double MODIFICATOR OddsRatio (double,double,double,double,double*,double*,double);
double MODIFICATOR Organization(double,double);
double MODIFICATOR Page (double*,long,long,double*);
double MODIFICATOR Pearson (double*,double*,long);
double MODIFICATOR PearsonX(double*,long,long,double*);
double MODIFICATOR PowerOf (double,double);
double MODIFICATOR PPV (double,double,double,double,double*,double*,double,double);
double MODIFICATOR Prevalence (double,double,double,double,double*,double*,double);
double MODIFICATOR Probit (double);
double MODIFICATOR Proportion (double*,long);
double MODIFICATOR ProportionDifference (double,double,double,double,double*,double*,double*,double,long);
double MODIFICATOR ProportionDifferencePair (double,double,double,double,double*,double*,double*,double,long);
double MODIFICATOR ProportionDifferencePairA (double,double,double,double,double*,double*,double*,double,long);
double MODIFICATOR PuriSen (double*,double*,long,long,long);
double MODIFICATOR PuriSenTamura (double*,double*,long,long,long);
double MODIFICATOR Quade (double*,long,long);
double MODIFICATOR QuartileLow(double*,long);
double MODIFICATOR QuartileUpp(double*,long);
double MODIFICATOR r8_uniform_01 (int*);
double MODIFICATOR RandomITest (double*,long,double*,long,double*);
double MODIFICATOR RandomTest (double*,long,double*,double*);
double MODIFICATOR RangeDensity (double,double,long);
double MODIFICATOR RangeDistribution(double,long);
double MODIFICATOR RangeStudentDensity (double,double,long,long);
double MODIFICATOR RangeStudentDistribution(double,long,long);
double MODIFICATOR Rank (double*,double*,long);
double MODIFICATOR RankAB (double*,double*,long);
double MODIFICATOR RankFriedman (double*,double*,long);
double MODIFICATOR RankKendall (double*,double*,long);
double MODIFICATOR RankKruskal (double*,double*,long);
double MODIFICATOR RelativeRisk (double,double,double,double,double*,double*,double);
double MODIFICATOR RelEntropy(double,double);
double MODIFICATOR RepeatedANOVA (double*,long,long);
double MODIFICATOR RosenblattTest (double*,long,double*,long,double*);
double MODIFICATOR SavageTest (double*,long,double*,long,double*);
double MODIFICATOR Sensitivity (double,double,double*,double*,double);
double MODIFICATOR MannWhitneyTest (double*,long,double*,long,double*);
double MODIFICATOR ShapiroFranciaTest (double*,long,double*);
double MODIFICATOR ShapiroWilkTest (double*,long,double*);
double MODIFICATOR SiegelTukeyTest (double*,long,double*,long,double*);
double MODIFICATOR SignTest (double*,long,double*,double*);
double MODIFICATOR SimonoffTsai(double*,long,long);
double MODIFICATOR Skewness (double*,long);
double MODIFICATOR SkewnessTest (double*,long,double*);
double MODIFICATOR SmirnovExactTest (double*,long,double*,long,double*);
double MODIFICATOR SmirnovGrubbsDistribution (long,double);
double MODIFICATOR SmirnovTest (double*,long,double*);
double MODIFICATOR SMMDensity (double,double,double,double,double);
double MODIFICATOR SMMDistribution(double,double,double);
double MODIFICATOR Specificity (double,double,double*,double*,double);
double MODIFICATOR StuartMaxwellTest (double*,long);
double MODIFICATOR StudentPairedTest (double*,long,double*,double*);
double MODIFICATOR StudentTest (double*,long,double*,double*);
double MODIFICATOR StudentUnpairedTest (double*,long,double*,long,double*);
double MODIFICATOR SuJohnsonDistribution (double,double,double,double,double);
double MODIFICATOR Sup (double*,double*,long);
double MODIFICATOR Surplus(double,double);
double MODIFICATOR Tanh(double);
double MODIFICATOR TDistribution (double,double);
double MODIFICATOR ThompsonDistribution (long,double);
double MODIFICATOR TietjenMooreDistribution(long,long);
double MODIFICATOR Trace (double*,long);
double MODIFICATOR Transgression (double*,long,double*,long);
double MODIFICATOR VanDerWaerdenTest (double*,long,double*,long,double*);
double MODIFICATOR Variance (double*,long);
double MODIFICATOR VasicekTest (double*,long,long,double*);
double MODIFICATOR VectorNorma (double*,long);
double MODIFICATOR VectorSumSQ (double*,long);
double MODIFICATOR Wald (double,double,double,double);
double MODIFICATOR WaldWolfowitzTest (double*,long,double*,long,double*,long);
double MODIFICATOR WaldWolfowitzExact (double*,long,double*,long,double*);
double MODIFICATOR WeibullAdd (double*,long,double,double,double);
double MODIFICATOR WelchTest (double*,long,double*,long,double*);
double MODIFICATOR WilcoxonExact (double*,long,double*,long,double*);
double MODIFICATOR WilcoxonSignedExact (double*,long,double*,double*);
double MODIFICATOR WilcoxonSignedTest (double*,long,double*,double*,long);
double MODIFICATOR WilcoxonTest (double*,long,double*,long,double*,long,long);
double MODIFICATOR Wilks(double*,long,long,long);
double MODIFICATOR YeoJohnson (double*,long);
double MODIFICATOR YeoJohnsonML (double*,long,double);
double MODIFICATOR YeoJohnsonTransform (double,double);
double MODIFICATOR Yule (double*,double*,long);
double MODIFICATOR ZellnerRevankar (double*,long);
double MODIFICATOR ZellnerRevankarML (double*,long,double);
double MODIFICATOR ZellnerRevankarTransform (double,double);
double MODIFICATOR Zelterman(double*,long,long);
double MODIFICATOR ZTest (double,double,double,double,double*,long);
double MODIFICATOR ZTestPair (double,double,double,double,double*,long);
long MODIFICATOR ANCOVA (double*,long*,long,double*,double*,double*,double*,double*,double*,double*,double*,double*,double*,double*,double*,double*);
long MODIFICATOR LikenessMatrixMixed(double*,long,long,long*,double*);
long MODIFICATOR BoxCoxMulti (double*,long,double*,long);
long MODIFICATOR CanonicalCorrelationAnalysis (double*,double*,long,long,long,long,double*,double*,double*,double*,double*);
void MODIFICATOR CorrelationMatrix (double*,double*,long,long);
long MODIFICATOR CDA (double*,long,long,long*,long,long*,double*,double*);
long MODIFICATOR CDAClass (double*,double*,double*,long,long,long*);
long MODIFICATOR Choleski (double*,double*,long);
long MODIFICATOR ClassesOptimal (double*,long);
long MODIFICATOR Combination (double*,long,long,double,double*);
long MODIFICATOR Conover (double*,long*,long,double*,double*,long);
long MODIFICATOR CovMatrix (double*,double*,long,long);
long MODIFICATOR CoxHazard (double*,double*,double*,double*,double*,long,long,long*);
long MODIFICATOR Duncan (double*,long*,long,double*,long*);
long MODIFICATOR Dunn (double*,long*,long,double*,double*,double*);
long MODIFICATOR DurbinWatsonBetaPars (double*,long,long,double*,double*);
long MODIFICATOR Eigen (double*,double*,long,double,long*);
long MODIFICATOR ExponentFunction (double*,double*,long,double*);
long MODIFICATOR ExpPowerFunction (double*,double*,long,double*);
long MODIFICATOR FarrarGlauber2 (double*,long,long,double*,double*,double*);
long MODIFICATOR FarrarGlauber3 (double*,long,long,double*,double*,double*,long*,long*);
long MODIFICATOR FDA (double*,long,long,long*,long,long*,double*);
long MODIFICATOR FDAClass (double*,double*,long,long,long*,double*);
long MODIFICATOR Gauss (double*,double*,long,long,double);
long MODIFICATOR GaussL(long double*,long double*,long,long,long double);
long MODIFICATOR Gower (double*,long,long,long*,double*);
long MODIFICATOR Gram (double*,long);
long MODIFICATOR HyperFunction (double*,double*,long,double*);
long MODIFICATOR InverseMatrix (double*,double*,long);
long MODIFICATOR Jacobi (double*,long,double,long*,double*);
long MODIFICATOR KemenyMean (double*,long,long,double*);
long MODIFICATOR KemenyMedian (double*,long,long,double*);
long MODIFICATOR KendallCoeff (double*,double*,double*,double*,long,double*);
long MODIFICATOR King (double*,long,long,long,long*,long*,double*);
long MODIFICATOR Kruskal(double*,long,long,long,double*,double*,long,long,long*,double*,double,double);
long MODIFICATOR LDA (double*,long,long,long*,long,long*,double*,double*,double*);
long MODIFICATOR LDAClass (double*,double*,double*,double*,long,long,long*,double*);
long MODIFICATOR LikenessMatrix (double*,double*,double*,long,long,long);
long MODIFICATOR LinearRegression (double*,double*,double*,long,long,double*,double*,double*);
long MODIFICATOR LogarithmFunction (double*,double*,long,double*);
long MODIFICATOR LogisticFunction (double*,double*,long,double*);
long MODIFICATOR LogisticRegression (double*,double*,double*,long,long,long*,double*);
long MODIFICATOR MANOVA (double*,long,long,double*,double*,double*,double*,double*);
long MODIFICATOR MaximumLikelihood (double*,long,double*,double*,long,long*);
long MODIFICATOR McQueen (double*,long,long,long,long,long*,double*);
long MODIFICATOR MeanRank (double*,long,long,double*);
long MODIFICATOR MinimumOf(double*,long);
long MODIFICATOR Parity (long);
long MODIFICATOR PieceLinearFunction (double*,double*,long,double*,double*);
long MODIFICATOR Point (double*,double*,long,double*);
long MODIFICATOR PoissonRegression (double*,double*,double*,long,long,long*,double*);
long MODIFICATOR PolynomialFunction (double*,double*,long,long,long,double*);
long MODIFICATOR PowerFunction (double*,double*,long,double*);
long MODIFICATOR PrincipalFactor (double*,long,double*,double*,long*);
long MODIFICATOR ProbitRegression (double*,double*,double*,long,long,long*,double*);
long MODIFICATOR PseudoInverseMatrix (double*,double*,long,long);
long MODIFICATOR QRDecomposition (double*,double*,double*,long,long);
long MODIFICATOR Reduction (double*,double*,long,long);
long MODIFICATOR ROC(double*,long,double*,long,double*,double*,double*,double*,double*);
long MODIFICATOR Scheffe (double*,long*,long,double*);
long MODIFICATOR ScheffePair (double*,long,long,double*,double*);
long MODIFICATOR ShimazakiShinomoto (double*,long,double*,double*);
long MODIFICATOR SingularValueDecomposition (double*,long,long,double*,double*,double*);
long MODIFICATOR Solves (double*,double*,long,long);
long MODIFICATOR Spearman (double*,double*,long,double*);
long MODIFICATOR Sturgess(long);
long MODIFICATOR Thurstone (double*,long,long,double*,double*);
long MODIFICATOR Thurstone27 (double*,long,double*);
long MODIFICATOR Torgerson (double*,long,double*,double*,long*);
long MODIFICATOR Tukey (double*,long,long,double*);
long MODIFICATOR TwoDegree (long);
long MODIFICATOR Varimax (double*,long,long);
long MODIFICATOR Ward (double*,long,long,long*,long*,double*);
long MODIFICATOR Weights(long,long,double*,double*);
void MODIFICATOR Antilex (long,long,double*,long,double*,double*);
void MODIFICATOR ArrayToArray (double*,double*,long);
void MODIFICATOR BoxCoxTransformMulti (double*,long,double*,double*);
void MODIFICATOR ClusterPresentKW(long*,long*,long*,long*,long,long);
void MODIFICATOR ClusterPresentMQ(long*,long*,long*,long);
void MODIFICATOR ColInterchange (double*,long,long,long);
void MODIFICATOR CombinationAdd (double*,long,long,double*,double,long*,double*,long);
void MODIFICATOR DifferenceMatrix(double*,double*,long,long,long);
void MODIFICATOR ExpFitting (double*,double*,long,double*,double*,long,double*,double*);
void MODIFICATOR ExponentialEstimate (double*,double*,long,double*);
void MODIFICATOR Fourier(double*,long,long,double*,double*);
void MODIFICATOR GammaEstimate (double*,double*,long,double*,double*);
void MODIFICATOR GammaFitting (double*,double*,long,double*,double*,long,double*,double*,double*);
void MODIFICATOR Generate (long*,long,long*,double*,double*,long*,long,double*,long*);
void MODIFICATOR GompertzEstimate (double*,double*,long,double*,double*);
void MODIFICATOR GompertzFitting (double*,double*,long,double*,double*,long,double*,double*,double*);
void MODIFICATOR Hazard(double*,double*,long,double*,double*,long*,double,double*,double*,double*);
void MODIFICATOR Hutcheson(double*,long,long,double*,long,long,double,double*,double*);
void MODIFICATOR Inter(double*,long,double*,double*);
void MODIFICATOR IntervalCorrelation(double,long,double,double*,double*);
void MODIFICATOR IntervalKendall(double*,double*,long,double,double,double*,double*,long*);
void MODIFICATOR LoglogisticFitting (double*,double*,long,double*,double*,long,double*,double*,double*);
void MODIFICATOR LognormalEstimate (double*,double*,long,double*,double*);
void MODIFICATOR LognormalFitting (double*,double*,long,double*,double*,long,double*,double*,double*);
void MODIFICATOR MatrixMinusMatrix (double*,double*,long);
void MODIFICATOR MatrixMinusMatrix2 (double*,double*,double*,long);
void MODIFICATOR MatrixMultiplyMatrix (double*,double*,double*,long,long,long);
void MODIFICATOR MatrixMultiplyMatrixT (double*,double*,double*,long,long,long);
void MODIFICATOR MatrixMultiplyNumber (double*,long,double);
void MODIFICATOR MatrixPlusMatrix (double*,double*,long);
void MODIFICATOR MatrixT (double*,double*,long,long);
void MODIFICATOR MatrixTMultiplyMatrix (double*,double*,double*,long,long,long);
void MODIFICATOR MatrixUnit (double*,long);
void MODIFICATOR MeanVector (double*,double*,long,long);
void MODIFICATOR MedianDifference(double*,long,double*,long,double,double*,double*,double*);
void MODIFICATOR MedianVar(double*,long,double*,double*,double*);
void MODIFICATOR MovingAverage(double*,double*,double*,double*,long,long,long,double);
void MODIFICATOR PercentOfVariance (double*,long,double*,double*,long);
void MODIFICATOR Pocket(double*,long,double*,double*,double*,long);
void MODIFICATOR PseudoMedian(double*,long,double,double*,double*,double*);
void MODIFICATOR RankSavage (double*,long);
void MODIFICATOR RankST (double*,double*,long);
void MODIFICATOR RayleighEstimate (double*,double*,long,double*);
void MODIFICATOR RayleighFitting (double*,double*,long,double*,double*,long,double*,double*);
void MODIFICATOR Reverse (long);
void MODIFICATOR ScalarMatrix (double*,double*,long);
void MODIFICATOR ScheirerRayHare (double*,double*,long,long,double*,long*);
void MODIFICATOR Somers(double*,long,long,double*,double*);
void MODIFICATOR SortArrayCommon (double*,double*,long);
void MODIFICATOR SortArrayCommonI (double*,long*,long);
void MODIFICATOR SortArrayDown (double*,long,long*);
void MODIFICATOR SortArrayDown0 (double*,long);
void MODIFICATOR SortArrayUp (double*,long,long*);
void MODIFICATOR SortArrayUp0 (double*,long);
void MODIFICATOR SortArrayUpI (double*,long);
void MODIFICATOR Survive(double*,double*,long,double*,double*,long*,double,double*,double*,double*);
void MODIFICATOR UnSortArray (double*,double*,long,long*);
void MODIFICATOR WeibullEstimate (double*,double*,long,double*,double*);
void MODIFICATOR WeibullFitting (double*,double*,long,double*,double*,long,double*,double*,double*);
double MODIFICATOR PagurovaDistribution (double,double,double,double);
double MODIFICATOR PagurovaTest (double*,long,double*,long,double*);
double MODIFICATOR InversePagurovaDistribution (double,double,double,double);
double MODIFICATOR Bravais (double*,double*,long);
long MODIFICATOR Kendall (double*,double*,long,double*);
double MODIFICATOR ChiSquareTest (double,double,double,double,double*,long);
void MODIFICATOR Median(double*,long,double,double*,double*,double*);
double MODIFICATOR MaximumOf (double*,long);

#ifdef EXCEL
  }
#endif

///////////////////////////////
// The function for inside call
double Bravais (double*,long,double*,long);
void FillUp (double*,long,double);
void FillUp (long*,long,long);
void FillUpCol (double*,long,long,double);
double Gamma (double,double);
double Rao (double*,double*,long);
double Rao (double*,long,double*,long);
double Combination (long,long);
double Pow (double,long);
double Pow (long,long);
double ChiSquareTest (double*,double*,long);
double MannWhitneyTest (double*,double*,long,long);
double MaximumOf (double*,long,long*);
double Kendall(double*,long,long,double*);
long Likeness (double*,double*,long,long,long);
double Likeness (double*,double*,long);
long VectorSum (long*,long);
double Median(double*,long);
long BFGS (double*,long,double*,long,double,long*,double,userFuncGrad,userFuncRight);
double BoxCoxMLMulti (double*,long,double*,long);
void Gradient (double*,long,double*,double*,long,double,userFuncRight);
long MaximumOf (long*,long);
